@@ -585,8 +585,10 @@ static void zap_sector(ext2_filsys fs, int sect, int nsect)
 		else {
 			magic = (unsigned int *) (buf + BSD_LABEL_OFFSET);
 			if ((*magic == BSD_DISKMAGIC) ||
-			    (*magic == BSD_MAGICDISK))
+			    (*magic == BSD_MAGICDISK)) {
+				free(buf);
 				return;
+			}
 		}
 	}
 
@@ -1577,7 +1579,9 @@ static void PRS(int argc, char *argv[])
 	int		use_bsize;
 	char		*newpath;
 	int		pathlen = sizeof(PATH_SET) + 1;
+#ifdef HAVE_BLKID_PROBE_GET_TOPOLOGY
 	struct device_param dev_param;
+#endif
 
 	if (oldpath)
 		pathlen += strlen(oldpath);
@@ -1969,18 +1973,8 @@ profile_error:
 	profile_get_integer(profile, "options", "proceed_delay", 0, 0,
 			    &proceed_delay);
 
-	/* The isatty() test is so we don't break existing scripts */
-	flags = CREATE_FILE;
-	if (isatty(0) && isatty(1) && !offset)
-		flags |= CHECK_FS_EXIST;
-	if (!quiet)
-		flags |= VERBOSE_CREATE;
-	if (fs_blocks_count == 0)
-		flags |= NO_SIZE;
-	else
+	if (fs_blocks_count)
 		explicit_fssize = 1;
-	if (!check_plausibility(device_name, flags, &is_device) && !force)
-		proceed_question(proceed_delay);
 
 	check_mount(device_name, force, _("filesystem"));
 
@@ -1989,15 +1983,9 @@ profile_error:
 		dev_size = fs_blocks_count;
 		retval = 0;
 	} else
-#ifndef _WIN32
 		retval = ext2fs_get_device_size2(device_name,
 						 EXT2_BLOCK_SIZE(&fs_param),
 						 &dev_size);
-#else
-		retval = ext2fs_get_device_size(device_name,
-						EXT2_BLOCK_SIZE(&fs_param),
-						&dev_size);
-#endif
 	if (retval && (retval != EXT2_ET_UNIMPLEMENTED)) {
 		com_err(program_name, retval, "%s",
 			_("while trying to determine filesystem size"));
@@ -2333,9 +2321,9 @@ profile_error:
 			device_name);
 	} else {
 		/* setting stripe/stride to blocksize is pointless */
-		if (dev_param.min_io > blocksize)
+		if (dev_param.min_io > (unsigned) blocksize)
 			fs_param.s_raid_stride = dev_param.min_io / blocksize;
-		if (dev_param.opt_io > blocksize) {
+		if (dev_param.opt_io > (unsigned) blocksize) {
 			fs_param.s_raid_stripe_width =
 						dev_param.opt_io / blocksize;
 		}
@@ -2652,6 +2640,17 @@ profile_error:
 
 	free(fs_type);
 	free(usage_types);
+
+	/* The isatty() test is so we don't break existing scripts */
+	flags = CREATE_FILE;
+	if (isatty(0) && isatty(1) && !offset)
+		flags |= CHECK_FS_EXIST;
+	if (!quiet)
+		flags |= VERBOSE_CREATE;
+	if (!explicit_fssize)
+		flags |= NO_SIZE;
+	if (!check_plausibility(device_name, flags, &is_device) && !force)
+		proceed_question(proceed_delay);
 }
 
 static int should_do_undo(const char *name)
@@ -2796,7 +2795,7 @@ static int mke2fs_discard_device(ext2_filsys fs)
 	struct ext2fs_numeric_progress_struct progress;
 	blk64_t blocks = ext2fs_blocks_count(fs->super);
 	blk64_t count = DISCARD_STEP_MB;
-	blk64_t cur;
+	blk64_t cur = 0;
 	int retval = 0;
 
 	/*
@@ -2804,10 +2803,9 @@ static int mke2fs_discard_device(ext2_filsys fs)
 	 * we do not print numeric progress resulting in failure
 	 * afterwards.
 	 */
-	retval = io_channel_discard(fs->io, 0, fs->blocksize);
+	retval = io_channel_discard(fs->io, 0, 1);
 	if (retval)
 		return retval;
-	cur = fs->blocksize;
 
 	count *= (1024 * 1024);
 	count /= fs->blocksize;
